@@ -28,8 +28,8 @@ namespace yaramod {
 class StringExpression : public Expression
 {
 public:
-	StringExpression(const std::string& id) { _id = _tokenStream->emplace_back(STRING_ID, id); }
-	StringExpression(std::string&& id) { _id = _tokenStream->emplace_back(STRING_ID, std::move(id)); }
+	StringExpression(const std::string& id) { _id = _tokenStream->emplace_back(TokenType::STRING_ID, id); }
+	StringExpression(std::string&& id) { _id = _tokenStream->emplace_back(TokenType::STRING_ID, std::move(id)); }
 	StringExpression(TokenIt id) : _id(id) {}
 
 	virtual VisitResult accept(Visitor* v) override
@@ -37,11 +37,14 @@ public:
 		return v->visit(this);
 	}
 
-	const std::string& getId() const { return _id->getString(); }
+	std::string getId() const { return _id->getPureText(); }
 
 	void setId(const std::string& id) { _id->setValue(id); }
 	void setId(std::string&& id) { _id->setValue(std::move(id)); }
 
+	virtual TokenIt getFirstTokenIt() const override { return _id; }
+	virtual TokenIt getLastTokenIt() const override { return _id; }
+
 	virtual std::string getText(const std::string& /*indent*/ = std::string{}) const override
 	{
 		return getId();
@@ -67,7 +70,7 @@ public:
 	template <typename Str>
 	StringWildcardExpression(Str&& id)
 	{
-		_id = _tokenStream->emplace_back(STRING_ID, std::forward<Str>(id));
+		_id = _tokenStream->emplace_back(TokenType::STRING_ID, std::forward<Str>(id));
 	}
 	StringWildcardExpression(TokenIt it) : _id(it) {}
 
@@ -81,6 +84,9 @@ public:
 	void setId(const std::string& id) { _id->setValue(id); }
 	void setId(std::string&& id) { _id->setValue(std::move(id)); }
 
+	virtual TokenIt getFirstTokenIt() const override { return _id; }
+	virtual TokenIt getLastTokenIt() const override { return _id; }
+
 	virtual std::string getText(const std::string& /*indent*/ = std::string{}) const override
 	{
 		return getId();
@@ -105,9 +111,9 @@ public:
 	StringAtExpression(const std::string& id, ExpPtr&& at)
 		: _at(std::forward<ExpPtr>(at))
 	{
-		_id = _tokenStream->emplace_back(STRING_ID, id);
-		_at_symbol = _tokenStream->emplace_back(OP_AT, "at");
-		_tokenStream->move_append(_at->getTokenStream());
+		_id = _tokenStream->emplace_back(TokenType::STRING_ID, id);
+		_at_symbol = _tokenStream->emplace_back(TokenType::OP_AT, "at");
+		_tokenStream->moveAppend(_at->getTokenStream());
 	}
 
 	template <typename ExpPtr>
@@ -123,7 +129,7 @@ public:
 		return v->visit(this);
 	}
 
-	const std::string& getId() const { return _id->getString(); }
+	std::string getId() const { return _id->getPureText(); }
 	const Expression::Ptr& getAtExpression() const { return _at; }
 
 	void setId(const std::string& id) { _id->setValue(id); }
@@ -131,6 +137,9 @@ public:
 	void setAtExpression(const Expression::Ptr& at) { _at = at; }
 	void setAtExpression(Expression::Ptr&& at) { _at = std::move(at); }
 
+	virtual TokenIt getFirstTokenIt() const override { return _id; }
+	virtual TokenIt getLastTokenIt() const override { return _at->getLastTokenIt(); }
+
 	virtual std::string getText(const std::string& indent = std::string{}) const override
 	{
 		return getId() + " " + _at_symbol->getString() + " " + _at->getText(indent);
@@ -156,10 +165,10 @@ public:
 	template <typename ExpPtr>
 	StringInRangeExpression(const std::string& id, ExpPtr&& range)
 	{
-		_id = _tokenStream->emplace_back(STRING_ID, id);
-		_in_symbol = _tokenStream->emplace_back(OP_IN, "in");
+		_id = _tokenStream->emplace_back(TokenType::STRING_ID, id);
+		_in_symbol = _tokenStream->emplace_back(TokenType::OP_IN, "in");
 		_range = std::forward<ExpPtr>(range);
-		_tokenStream->move_append(_range->getTokenStream());
+		_tokenStream->moveAppend(_range->getTokenStream());
 	}
 
 	template <typename ExpPtr>
@@ -175,7 +184,7 @@ public:
 		return v->visit(this);
 	}
 
-	const std::string& getId() const { return _id->getString(); }
+	std::string getId() const { return _id->getPureText(); }
 	const Expression::Ptr& getRangeExpression() const { return _range; }
 
 	void setId(const std::string& id) { _id->setValue(id); }
@@ -183,6 +192,9 @@ public:
 	void setRangeExpression(const Expression::Ptr& range) { _range = range; }
 	void setRangeExpression(Expression::Ptr&& range) { _range = std::move(range); }
 
+	virtual TokenIt getFirstTokenIt() const override { return _id; }
+	virtual TokenIt getLastTokenIt() const override { return _range->getLastTokenIt(); }
+
 	virtual std::string getText(const std::string& indent = std::string{}) const override
 	{
 		return getId() + " " + _in_symbol->getString() + " " + _range->getText(indent);
@@ -211,7 +223,7 @@ public:
 	template <typename Str>
 	StringCountExpression(Str&& id)
 	{
-		_id = _tokenStream->emplace_back(STRING_COUNT, std::forward<Str>(id));
+		_id = _tokenStream->emplace_back(TokenType::STRING_COUNT, std::forward<Str>(id));
 	}
 
 	virtual VisitResult accept(Visitor* v) override
@@ -219,14 +231,20 @@ public:
 		return v->visit(this);
 	}
 
-	const std::string& getId() const { return _id->getString(); }
+	std::string getId() const { return _id->getPureText(); }
 
 	void setId(const std::string& id) { _id->setValue(id); }
 	void setId(std::string&& id) { _id->setValue(std::move(id)); }
 
+	virtual TokenIt getFirstTokenIt() const override { return _id; }
+	virtual TokenIt getLastTokenIt() const override { return _id; }
+
 	virtual std::string getText(const std::string& /*indent*/ = std::string{}) const override
 	{
-		return _id->getString();
+		auto output = getId();
+		assert(output != std::string() && "String id must be non-empty.");
+		output[0] = '#';
+		return output;
 	}
 
 private:
@@ -259,13 +277,13 @@ public:
 	template <typename Str>
 	StringOffsetExpression(Str&& id)
 	{
-		_id = _tokenStream->emplace_back(STRING_OFFSET, std::forward<Str>(id));
+		_id = _tokenStream->emplace_back(TokenType::STRING_OFFSET, std::forward<Str>(id));
 	}
 	template <typename Str, typename ExpPtr>
 	StringOffsetExpression(Str&& id, ExpPtr&& expr)
 		: _expr(std::forward<ExpPtr>(expr))
 	{
-		_id = _tokenStream->emplace_back(STRING_OFFSET, std::forward<Str>(id));
+		_id = _tokenStream->emplace_back(TokenType::STRING_OFFSET, std::forward<Str>(id));
 	}
 
 	virtual VisitResult accept(Visitor* v) override
@@ -273,7 +291,7 @@ public:
 		return v->visit(this);
 	}
 
-	const std::string& getId() const { return _id->getString(); }
+	std::string getId() const { return _id->getPureText(); }
 	const Expression::Ptr& getIndexExpression() const { return _expr; }
 
 	void setId(const std::string& id) { _id->setValue(id); }
@@ -283,9 +301,15 @@ public:
 
 	virtual std::string getText(const std::string& indent = std::string{}) const override
 	{
-		return _expr ? getId() + '[' + _expr->getText(indent) + ']' : getId();
+		auto prefix = getId();
+		assert(prefix != std::string() && "String id must be non-empty.");
+		prefix[0] = '@';
+		return _expr ? prefix + '[' + _expr->getText(indent) + ']' : prefix;
 	}
 
+	virtual TokenIt getFirstTokenIt() const override { return _id; }
+	virtual TokenIt getLastTokenIt() const override { return _expr ? _expr->getLastTokenIt() : _id; }
+
 private:
 	TokenIt _id; ///< Identifier of the string
 	Expression::Ptr _expr; ///< Index expression if any
@@ -317,20 +341,21 @@ public:
 	template <typename Str>
 	StringLengthExpression(Str&& id)
 	{
-		_id = _tokenStream->emplace_back(STRING_LENGTH, std::forward<Str>(id));
+		_id = _tokenStream->emplace_back(TokenType::STRING_LENGTH, std::forward<Str>(id));
 	}
 	template <typename Str, typename ExpPtr>
 	StringLengthExpression(Str&& id, ExpPtr&& expr)
 		: _expr(std::forward<ExpPtr>(expr))
 	{
-		_id = _tokenStream->emplace_back(STRING_LENGTH, std::forward<Str>(id));
+		_id = _tokenStream->emplace_back(TokenType::STRING_LENGTH, std::forward<Str>(id));
 	}
+
 	virtual VisitResult accept(Visitor* v) override
 	{
 		return v->visit(this);
 	}
 
-	const std::string& getId() const { return _id->getString(); }
+	std::string getId() const { return _id->getPureText(); }
 	const Expression::Ptr& getIndexExpression() const { return _expr; }
 
 	void setId(const std::string& id) { _id->setValue(id); }
@@ -338,8 +363,14 @@ public:
 	void setIndexExpression(const Expression::Ptr& expr) { _expr = expr; }
 	void setIndexExpression(Expression::Ptr&& expr) { _expr = std::move(expr); }
 
+	virtual TokenIt getFirstTokenIt() const override { return _id; }
+	virtual TokenIt getLastTokenIt() const override { return _expr ? _expr->getLastTokenIt() : _id; }
+
 	virtual std::string getText(const std::string& indent = std::string{}) const override
 	{
+		auto prefix = getId();
+		assert(prefix != std::string() && "String id must be non-empty.");
+		prefix[0] = '!';
 		return _expr ? getId() + '[' + _expr->getText(indent) + ']' : getId();
 	}
 
@@ -356,7 +387,7 @@ class UnaryOpExpression : public Expression
 public:
 	virtual std::string getText(const std::string& indent = std::string{}) const override
 	{
-		if (_op->getType() == NOT)
+		if (_op->getType() == TokenType::NOT)
 			return _op->getString() + " " + _expr->getText(indent);
 		else
 			return _op->getString() + _expr->getText(indent);
@@ -380,6 +411,8 @@ protected:
 	{
 		_op = _tokenStream->emplace_back(type, op);
 	}
+	virtual TokenIt getFirstTokenIt() const override { return _op; }
+	virtual TokenIt getLastTokenIt() const override { return _expr->getLastTokenIt(); }
 
 private:
 	TokenIt _op; ///< Unary operation symbol, std::string
@@ -459,9 +492,13 @@ public:
 		return _left->getText(indent) + ' ' + _op->getString() + (_linebreak ? "\n" + indent : " ") + _right->getText(indent);
 	}
 
+	TokenIt getOperator() const { return _op; }
 	const Expression::Ptr& getLeftOperand() const { return _left; }
 	const Expression::Ptr& getRightOperand() const { return _right; }
 
+	virtual TokenIt getFirstTokenIt() const override { return _left->getFirstTokenIt(); }
+	virtual TokenIt getLastTokenIt() const override { return _right->getLastTokenIt(); }
+
 	void setLeftOperand(const Expression::Ptr& left) { _left = left; }
 	void setLeftOperand(Expression::Ptr&& left) { _left = std::move(left); }
 	void setRightOperand(const Expression::Ptr& right) { _right = right; }
@@ -964,6 +1001,11 @@ public:
 	{
 	}
 
+	const std::string& getId() const { return _id->getString(); }
+
+	void setId(const std::string& id) { _id->setValue(id); }
+	void setId(std::string&& id) { _id->setValue(std::move(id)); }
+
 	virtual VisitResult accept(Visitor* v) override
 	{
 		return v->visit(this);
@@ -979,6 +1021,9 @@ public:
 		return ss.str();
 	}
 
+	virtual TokenIt getFirstTokenIt() const override { return _for; }
+	virtual TokenIt getLastTokenIt() const override { return _right_bracket; }
+
 private:
 	TokenIt _id; ///< Iterating identifier
 	TokenIt _for; ///< TokenIt of 'for'
@@ -1022,6 +1067,9 @@ public:
 			<< _left_bracket->getString() << " " << _expr->getText(indent) << " " << _right_bracket->getString();
 		return ss.str();
 	}
+
+	virtual TokenIt getFirstTokenIt() const override { return _for; }
+	virtual TokenIt getLastTokenIt() const override { return _right_bracket; }
 private:
 	TokenIt _for;
 	TokenIt _left_bracket;
@@ -1058,6 +1106,9 @@ public:
 	{
 		return _forExpr->getText(indent) + " " + _of_in->getString() + " " + _set->getText(indent);
 	}
+
+	virtual TokenIt getFirstTokenIt() const override { return _forExpr->getFirstTokenIt(); }
+	virtual TokenIt getLastTokenIt() const override { return _set->getLastTokenIt(); }
 };
 
 /**
@@ -1107,6 +1158,9 @@ public:
 
 	const std::vector<Expression::Ptr>& getElements() const { return _elements; }
 
+	virtual TokenIt getFirstTokenIt() const override { return _left_bracket; }
+	virtual TokenIt getLastTokenIt() const override { return _right_bracket; }
+
 	void setElements(const std::vector<Expression::Ptr>& elements)
 	{
 		_elements = elements;
@@ -1160,6 +1214,9 @@ public:
 	const Expression::Ptr& getLow() const { return _low; }
 	const Expression::Ptr& getHigh() const { return _high; }
 
+	virtual TokenIt getFirstTokenIt() const override { return _left_bracket; }
+	virtual TokenIt getLastTokenIt() const override { return _right_bracket; }
+
 	void setLow(const Expression::Ptr& low) { _low = low; }
 	void setLow(Expression::Ptr&& low) { _low = std::move(low); }
 	void setHigh(const Expression::Ptr& high) { _high = high; }
@@ -1189,11 +1246,6 @@ public:
 	/**
 	 * Constructors
 	 */
-	IdExpression(const std::shared_ptr<Symbol>& symbol)
-		: _symbol(symbol)
-	{
-	}
-
 	IdExpression(TokenIt symbolToken)
 		: _symbol(symbolToken->getSymbol())
 		, _symbolToken(symbolToken)
@@ -1216,16 +1268,28 @@ public:
 		return _symbol;
 	}
 
+	TokenIt getSymbolToken() const
+	{
+		return _symbolToken;
+	}
+
+	virtual TokenIt getFirstTokenIt() const override { return _symbolToken; }
+	virtual TokenIt getLastTokenIt() const override { return _symbolToken; }
+
 	void setSymbol(const std::shared_ptr<Symbol>& symbol)
 	{
 		_symbol = symbol;
-		if (_symbolToken)
-			_symbolToken.value()->setValue(_symbol, _symbol->getName());
+		_symbolToken->setValue(_symbol);
 	}
 
 protected:
+	IdExpression(const std::shared_ptr<Symbol>& symbol)
+		: _symbol(symbol)
+	{
+	}
+
 	std::shared_ptr<Symbol> _symbol; ///< Symbol of the identifier
-	std::optional<TokenIt> _symbolToken; ///< Token of the
+	TokenIt _symbolToken; ///< Token of the identifier
 };
 
 /**
@@ -1263,6 +1327,9 @@ public:
 
 	const Expression::Ptr& getStructure() const { return _structure; }
 
+	virtual TokenIt getFirstTokenIt() const override { return _structure->getFirstTokenIt(); }
+	virtual TokenIt getLastTokenIt() const override { return _symbolToken; }
+
 	void setStructure(const Expression::Ptr& structure) { _structure = structure; }
 	void setStructure(Expression::Ptr&& structure) { _structure = std::move(structure); }
 
@@ -1292,10 +1359,11 @@ public:
 		, _accessor(std::forward<ExpPtr2>(accessor))
 		, _right_bracket(right_bracket)
 	{
+		_symbolToken = std::static_pointer_cast<const IdExpression>(_array)->getSymbolToken();
 	}
 	template <typename ExpPtr1, typename ExpPtr2>
 	ArrayAccessExpression(ExpPtr1&& array, TokenIt left_bracket, ExpPtr2&& accessor, TokenIt right_bracket)
-		: IdExpression(std::static_pointer_cast<const IdExpression>(array)->getSymbol())
+		: IdExpression(std::static_pointer_cast<const IdExpression>(array)->getSymbolToken())
 		, _array(std::forward<ExpPtr1>(array))
 		, _left_bracket(left_bracket)
 		, _accessor(std::forward<ExpPtr2>(accessor))
@@ -1316,6 +1384,9 @@ public:
 	const Expression::Ptr& getArray() const { return _array; }
 	const Expression::Ptr& getAccessor() const { return _accessor; }
 
+	virtual TokenIt getFirstTokenIt() const override { return _array->getFirstTokenIt(); }
+	virtual TokenIt getLastTokenIt() const override { return _right_bracket; }
+
 	void setArray(const Expression::Ptr& array) { _array = array; }
 	void setArray(Expression::Ptr&& array) { _array = std::move(array); }
 	void setAccessor(const Expression::Ptr& accessor) { _accessor = accessor; }
@@ -1342,7 +1413,7 @@ class FunctionCallExpression : public IdExpression
 public:
 	template <typename ExpPtr, typename ExpPtrVector>
 	FunctionCallExpression(ExpPtr&& func, TokenIt left_bracket, ExpPtrVector&& args, TokenIt right_bracket)
-		: IdExpression(std::static_pointer_cast<const IdExpression>(func)->getSymbol())
+		: IdExpression(std::static_pointer_cast<const IdExpression>(func)->getSymbolToken())
 		, _func(std::forward<ExpPtr>(func))
 		, _left_bracket(left_bracket)
 		, _args(std::forward<ExpPtrVector>(args))
@@ -1378,6 +1449,9 @@ public:
 	const Expression::Ptr& getFunction() const { return _func; }
 	const std::vector<Expression::Ptr>& getArguments() const { return _args; }
 
+	virtual TokenIt getFirstTokenIt() const override { return _func->getFirstTokenIt(); }
+	virtual TokenIt getLastTokenIt() const override { return _right_bracket; }
+
 	void setFunction(const Expression::Ptr& func) { _func = func; }
 	void setFunction(Expression::Ptr&& func) { _func = std::move(func); }
 	void setArguments(const std::vector<Expression::Ptr>& args) { _args = args; }
@@ -1400,7 +1474,7 @@ class LiteralExpression : public Expression
 public:
 	using LiteralType = T;
 
-	LiteralExpression() = default;
+	LiteralExpression() : _valid(false) {}
 	LiteralExpression(TokenIt value) : _value(value) {}
 	LiteralExpression(const std::shared_ptr<TokenStream>& ts, TokenIt value)
 		: Expression(ts)
@@ -1412,21 +1486,24 @@ public:
 
 	virtual std::string getText(const std::string& /*indent*/ = std::string{}) const override
 	{
-		if (_value.has_value()){
-			return _value.value()->getText();
-		}
+		if (_valid)
+			return _value->getText();
 		else
 			return std::string();
 	}
 
+	virtual TokenIt getFirstTokenIt() const override { return _value; }
+	virtual TokenIt getLastTokenIt() const override { return _value; }
+
 	void clear()
 	{
-		if (_value.has_value())
-			_tokenStream->erase(_value.value());
+		if (_valid)
+			_tokenStream->erase(_value);
 	}
 
 protected:
-	std::optional<TokenIt> _value; ///< Value of the literal
+	bool _valid = true; ///< Set if _value is valid
+	TokenIt _value; ///< Value of the literal
 };
 
 /**
@@ -1450,9 +1527,10 @@ public:
 		: LiteralExpression<bool>()
 	{
 		if (value)
-			_value = _tokenStream->emplace_back(BOOL_TRUE, value, "true");
+			_value = _tokenStream->emplace_back(TokenType::BOOL_TRUE, value, "true");
 		else
-			_value = _tokenStream->emplace_back(BOOL_FALSE, value, "false");
+			_value = _tokenStream->emplace_back(TokenType::BOOL_FALSE, value, "false");
+		_valid = true;
 	}
 
 	BoolLiteralExpression(const std::shared_ptr<TokenStream>& ts, TokenIt value)
@@ -1462,7 +1540,7 @@ public:
 
 	virtual LiteralType getValue() const override
 	{
-		return _value.value()->getBool();
+		return _value->getBool();
 	}
 
 	virtual VisitResult accept(Visitor* v) override
@@ -1494,7 +1572,7 @@ public:
 
 	virtual LiteralType getValue() const override
 	{
-		return _value.value()->getString();
+		return _value->getString();
 	}
 
 	virtual VisitResult accept(Visitor* v) override
@@ -1528,7 +1606,7 @@ public:
 
 	virtual LiteralType getValue() const override
 	{
-		return _value.value()->getUInt();
+		return _value->getUInt();
 	}
 
 	virtual VisitResult accept(Visitor* v) override
@@ -1562,7 +1640,7 @@ public:
 
 	virtual LiteralType getValue() const override
 	{
-		return _value.value()->getFloat();
+		return _value->getFloat();
 	}
 
 	virtual VisitResult accept(Visitor* v) override
@@ -1582,6 +1660,9 @@ public:
 		return _keyword->getString();
 	}
 
+	virtual TokenIt getFirstTokenIt() const override { return _keyword; }
+	virtual TokenIt getLastTokenIt() const override { return _keyword; }
+
 protected:
 	KeywordExpression() = default;
 	KeywordExpression(TokenIt keyword)
@@ -1589,6 +1670,12 @@ protected:
 	{
 		assert(keyword->isString());
 	}
+	KeywordExpression(const std::shared_ptr<TokenStream>& ts, TokenIt keyword)
+		: Expression(ts) 
+		, _keyword(keyword)
+	{
+		assert(keyword->isString());
+	}
 	TokenIt _keyword; ///< Keyword
 };
 
@@ -1608,6 +1695,10 @@ public:
 		: KeywordExpression(t)
 	{
 	}
+	FilesizeExpression(const std::shared_ptr<TokenStream>& ts, TokenIt t)
+		: KeywordExpression(ts, t)
+	{
+	}
 
 	virtual VisitResult accept(Visitor* v) override
 	{
@@ -1631,6 +1722,10 @@ public:
 		: KeywordExpression(t)
 	{
 	}
+	EntrypointExpression(const std::shared_ptr<TokenStream>& ts, TokenIt t)
+		: KeywordExpression(ts, t)
+	{
+	}
 
 	virtual VisitResult accept(Visitor* v) override
 	{
@@ -1655,6 +1750,10 @@ public:
 		: KeywordExpression(t)
 	{
 	}
+	AllExpression(const std::shared_ptr<TokenStream>& ts, TokenIt t)
+		: KeywordExpression(ts, t)
+	{
+	}
 
 	virtual VisitResult accept(Visitor* v) override
 	{
@@ -1679,6 +1778,10 @@ public:
 		: KeywordExpression(t)
 	{
 	}
+	AnyExpression(const std::shared_ptr<TokenStream>& ts, TokenIt t)
+		: KeywordExpression(ts, t)
+	{
+	}
 
 	virtual VisitResult accept(Visitor* v) override
 	{
@@ -1703,6 +1806,10 @@ public:
 		: KeywordExpression(t)
 	{
 	}
+	ThemExpression(const std::shared_ptr<TokenStream>& ts, TokenIt t)
+		: KeywordExpression(ts, t)
+	{
+	}
 
 	virtual VisitResult accept(Visitor* v) override
 	{
@@ -1758,6 +1865,9 @@ public:
 
 	const Expression::Ptr& getEnclosedExpression() const { return _expr; }
 
+	virtual TokenIt getFirstTokenIt() const override { return _left_bracket; }
+	virtual TokenIt getLastTokenIt() const override { return _right_bracket; }
+
 	void setEnclosedExpression(const Expression::Ptr& expr) { _expr = expr; }
 	void setEnclosedExpression(Expression::Ptr&& expr) { _expr = std::move(expr); }
 
@@ -1812,6 +1922,9 @@ public:
 	const std::string& getFunction() const { return _func->getString(); }
 	const Expression::Ptr& getArgument() const { return _expr; }
 
+	virtual TokenIt getFirstTokenIt() const override { return _func; }
+	virtual TokenIt getLastTokenIt() const override { return _right_bracket; }
+
 	void setFunction(const std::string& func) { _func->setValue(func); }
 	void setFunction(std::string&& func) { _func->setValue(std::move(func)); }
 	void setArgument(const Expression::Ptr& expr) { _expr = expr; }
@@ -1857,6 +1970,9 @@ public:
 
 	const std::shared_ptr<String>& getRegexpString() const { return _regexp; }
 
+	virtual TokenIt getFirstTokenIt() const override { return _regexp->getFirstTokenIt(); }
+	virtual TokenIt getLastTokenIt() const override { return _regexp->getLastTokenIt(); }
+
 	void setRegexpString(const std::shared_ptr<String>& regexp) { _regexp = regexp; }
 	void setRegexpString(std::shared_ptr<String>&& regexp) { _regexp = std::move(regexp); }
 
