@@ -75,7 +75,7 @@ Expression::Ptr YaraExpressionBuilder::get() const
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator!()
 {
-	TokenIt token = _tokenStream->emplace(_tokenStream->begin(), NOT, "not");
+	TokenIt token = _tokenStream->emplace(_tokenStream->begin(), TokenType::NOT, "not");
 	_expr = std::make_shared<NotExpression>(token, std::move(_expr));
 	setType(Expression::Type::Int);
 	return *this;
@@ -88,7 +88,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator!()
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator~()
 {
-	TokenIt token = _tokenStream->emplace(_tokenStream->begin(), BITWISE_NOT, "~");
+	TokenIt token = _tokenStream->emplace(_tokenStream->begin(), TokenType::BITWISE_NOT, "~");
 	_expr = std::make_shared<BitwiseNotExpression>(token, std::move(_expr));
 	setType(Expression::Type::Bool);
 	return *this;
@@ -101,7 +101,7 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator~()
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator-()
 {
-	TokenIt token = _tokenStream->emplace(_tokenStream->begin(), UNARY_MINUS, "-");
+	TokenIt token = _tokenStream->emplace(_tokenStream->begin(), TokenType::UNARY_MINUS, "-");
 	_expr = std::make_shared<UnaryMinusExpression>(token, std::move(_expr));
 	return *this;
 }
@@ -115,8 +115,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator-()
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator&&(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(AND, "and");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::AND, "and");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<AndExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -132,8 +132,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator&&(const YaraExpressionBui
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator||(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(OR, "or");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::OR, "or");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<OrExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -149,8 +149,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator||(const YaraExpressionBui
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator<(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(LT, "<");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::LT, "<");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<LtExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -166,8 +166,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator<(const YaraExpressionBuil
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator<=(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(LE, "<=");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::LE, "<=");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<LeExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -183,8 +183,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator<=(const YaraExpressionBui
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator>(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(GT, ">");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::GT, ">");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<GtExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -200,8 +200,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator>(const YaraExpressionBuil
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator>=(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(GE, ">=");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::GE, ">=");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<GeExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -217,8 +217,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator>=(const YaraExpressionBui
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator==(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(EQ, "==");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::EQ, "==");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<EqExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -234,8 +234,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator==(const YaraExpressionBui
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator!=(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(NEQ, "!=");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::NEQ, "!=");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<NeqExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -253,8 +253,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator+(const YaraExpressionBuil
 {
 	bool will_be_float = _expr->isFloat() || other._expr->isFloat();
 
-	TokenIt token = _tokenStream->emplace_back(PLUS, "+");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::PLUS, "+");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<PlusExpression>(std::move(_expr), token, other.get());
 	setType(will_be_float ? Expression::Type::Float : Expression::Type::Int);
@@ -272,8 +272,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator-(const YaraExpressionBuil
 {
 	bool will_be_float = _expr->isFloat() || other._expr->isFloat();
 
-	TokenIt token = _tokenStream->emplace_back(MINUS, "-");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::MINUS, "-");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<MinusExpression>(std::move(_expr), token, other.get());
 	setType(will_be_float ? Expression::Type::Float : Expression::Type::Int);
@@ -291,8 +291,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator*(const YaraExpressionBuil
 {
 	bool will_be_float = _expr->isFloat() || other._expr->isFloat();
 
-	TokenIt token = _tokenStream->emplace_back(MULTIPLY, "*");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::MULTIPLY, "*");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<MultiplyExpression>(std::move(_expr), token, other.get());
 	setType(will_be_float ? Expression::Type::Float : Expression::Type::Int);
@@ -310,8 +310,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator/(const YaraExpressionBuil
 {
 	bool will_be_float = _expr->isFloat() || other._expr->isFloat();
 
-	TokenIt token = _tokenStream->emplace_back(DIVIDE, "\\");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::DIVIDE, "\\");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<DivideExpression>(std::move(_expr), token, other.get());
 	setType(will_be_float ? Expression::Type::Float : Expression::Type::Int);
@@ -329,8 +329,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator%(const YaraExpressionBuil
 {
 	bool will_be_float = _expr->isFloat() || other._expr->isFloat();
 
-	TokenIt token = _tokenStream->emplace_back(MODULO, "%");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::MODULO, "%");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<ModuloExpression>(std::move(_expr), token, other.get());
 	setType(will_be_float ? Expression::Type::Float : Expression::Type::Int);
@@ -346,8 +346,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator%(const YaraExpressionBuil
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator^(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(BITWISE_XOR, "^");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::BITWISE_XOR, "^");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<BitwiseXorExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Int);
@@ -363,8 +363,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator^(const YaraExpressionBuil
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator&(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(BITWISE_AND, "&");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::BITWISE_AND, "&");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<BitwiseAndExpression>(std::move(_expr), token, other.get());
 
@@ -381,8 +381,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator&(const YaraExpressionBuil
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator|(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(BITWISE_OR, "|");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::BITWISE_OR, "|");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<BitwiseOrExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Int);
@@ -398,8 +398,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator|(const YaraExpressionBuil
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator<<(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(SHIFT_LEFT, "<<");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::SHIFT_LEFT, "<<");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<ShiftLeftExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Int);
@@ -415,8 +415,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator<<(const YaraExpressionBui
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator>>(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(SHIFT_RIGHT, ">>");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::SHIFT_RIGHT, ">>");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<ShiftRightExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Int);
@@ -432,14 +432,14 @@ YaraExpressionBuilder& YaraExpressionBuilder::operator>>(const YaraExpressionBui
  */
 YaraExpressionBuilder& YaraExpressionBuilder::call(const std::vector<YaraExpressionBuilder>& args)
 {
-	TokenIt lb = _tokenStream->emplace_back(FUNCTION_CALL_LP, "(");
+	TokenIt lb = _tokenStream->emplace_back(TokenType::FUNCTION_CALL_LP, "(");
 	for (std::size_t i = 0; i < args.size(); ++i)
 	{
-		_tokenStream->move_append(args[i].getTokenStream());
+		_tokenStream->moveAppend(args[i].getTokenStream());
 		if (i < args.size() - 1)
-			_tokenStream->emplace_back(COMMA, ",");
+			_tokenStream->emplace_back(TokenType::COMMA, ",");
 	}
-	TokenIt rb = _tokenStream->emplace_back(FUNCTION_CALL_RP, ")");
+	TokenIt rb = _tokenStream->emplace_back(TokenType::FUNCTION_CALL_RP, ")");
 
 	std::vector<Expression::Ptr> exprArgs;
 	std::for_each(args.begin(), args.end(), [&exprArgs](const YaraExpressionBuilder& expr) {
@@ -504,8 +504,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::comment(const std::string& message
  */
 YaraExpressionBuilder& YaraExpressionBuilder::contains(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(CONTAINS, "contains");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::CONTAINS, "contains");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<ContainsExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -521,8 +521,8 @@ YaraExpressionBuilder& YaraExpressionBuilder::contains(const YaraExpressionBuild
  */
 YaraExpressionBuilder& YaraExpressionBuilder::matches(const YaraExpressionBuilder& other)
 {
-	TokenIt token = _tokenStream->emplace_back(MATCHES, "matches");
-	_tokenStream->move_append(other.getTokenStream());
+	TokenIt token = _tokenStream->emplace_back(TokenType::MATCHES, "matches");
+	_tokenStream->moveAppend(other.getTokenStream());
 
 	_expr = std::make_shared<MatchesExpression>(std::move(_expr), token, other.get());
 	setType(Expression::Type::Bool);
@@ -538,11 +538,10 @@ YaraExpressionBuilder& YaraExpressionBuilder::matches(const YaraExpressionBuilde
  */
 YaraExpressionBuilder& YaraExpressionBuilder::access(const std::string& attr) // pe.attr
 {
-	TokenIt dotIt = _tokenStream->emplace_back(DOT, ".");
+	TokenIt dotIt = _tokenStream->emplace_back(TokenType::DOT, ".");
 	const std::shared_ptr<Symbol>& symbol = std::make_shared<ValueSymbol>(attr, Expression::Type::Object);
 	Expression::Type type = symbol->getDataType();
-	const std::string& symbolName = symbol->getName();
-	TokenIt symbolIt = _tokenStream->emplace_back(ID, std::move(symbol), symbolName);
+	TokenIt symbolIt = _tokenStream->emplace_back(TokenType::ID, std::move(symbol));
 
 	_expr = std::make_shared<StructAccessExpression>(std::move(_expr), dotIt, symbolIt);
 	setType(type);
@@ -558,9 +557,9 @@ YaraExpressionBuilder& YaraExpressionBuilder::access(const std::string& attr) //
  */
 YaraExpressionBuilder& YaraExpressionBuilder::operator[](const YaraExpressionBuilder& other)
 {
-	TokenIt lsqb = _tokenStream->emplace_back(LSQB, "[");
-	_tokenStream->move_append(other.getTokenStream());
-	TokenIt rsqb = _tokenStream->emplace_back(RSQB, "]");
+	TokenIt lsqb = _tokenStream->emplace_back(TokenType::LSQB, "[");
+	_tokenStream->moveAppend(other.getTokenStream());
+	TokenIt rsqb = _tokenStream->emplace_back(TokenType::RSQB, "]");
 
 	_expr = std::make_shared<ArrayAccessExpression>(std::move(_expr), lsqb, std::move(other.get()), rsqb);
 	setType(Expression::Type::Undefined);
@@ -641,9 +640,9 @@ YaraExpressionBuilder& YaraExpressionBuilder::readUInt32(IntFunctionEndianness e
 
 YaraExpressionBuilder& YaraExpressionBuilder::readIntegerFunction(const std::string& function_name)
 {
-	TokenIt lb = _tokenStream->emplace(_tokenStream->begin(), LP, "(");
-	TokenIt func = _tokenStream->emplace(_tokenStream->begin(), INTEGER_FUNCTION, std::move(function_name));
-	TokenIt rb = _tokenStream->emplace_back(RP, ")");
+	TokenIt lb = _tokenStream->emplace(_tokenStream->begin(), TokenType::LP, "(");
+	TokenIt func = _tokenStream->emplace(_tokenStream->begin(), TokenType::INTEGER_FUNCTION, std::move(function_name));
+	TokenIt rb = _tokenStream->emplace_back(TokenType::RP, ")");
 
 	_expr = std::make_shared<IntFunctionExpression>(func, lb, std::move(_expr), rb);
 	setType(Expression::Type::Int);
@@ -674,7 +673,7 @@ YaraExpressionBuilder intVal(std::int64_t value, IntMultiplier mult)
 			break;
 	}
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(INTEGER, value, std::move(strValue));
+	TokenIt token = ts->emplace_back(TokenType::INTEGER, value, std::move(strValue));
 	auto expression = std::make_shared<IntLiteralExpression>(token);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Int);
 }
@@ -702,7 +701,7 @@ YaraExpressionBuilder uintVal(std::uint64_t value, IntMultiplier mult)
 YaraExpressionBuilder hexIntVal(std::uint64_t value)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(INTEGER, value, numToStr(value, std::hex, true));
+	TokenIt token = ts->emplace_back(TokenType::INTEGER, value, numToStr(value, std::hex, true));
 	auto expression = std::make_shared<IntLiteralExpression>(token);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Int);
 }
@@ -717,7 +716,7 @@ YaraExpressionBuilder hexIntVal(std::uint64_t value)
 YaraExpressionBuilder doubleVal(double value)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(DOUBLE, value, numToStr(value));
+	TokenIt token = ts->emplace_back(TokenType::DOUBLE, value, numToStr(value));
 	auto expression = std::make_shared<DoubleLiteralExpression>(token);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Float);
 }
@@ -732,7 +731,7 @@ YaraExpressionBuilder doubleVal(double value)
 YaraExpressionBuilder stringVal(const std::string& value)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(STRING_LITERAL, escapeString(value));
+	TokenIt token = ts->emplace_back(TokenType::STRING_LITERAL, escapeString(value));
 	token->markEscaped();
 	auto expression = std::make_shared<StringLiteralExpression>(token);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::String);
@@ -748,7 +747,7 @@ YaraExpressionBuilder stringVal(const std::string& value)
 YaraExpressionBuilder boolVal(bool value)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(value ? BOOL_TRUE : BOOL_FALSE, value);
+	TokenIt token = ts->emplace_back(value ? TokenType::BOOL_TRUE : TokenType::BOOL_FALSE, value);
 	auto expression = std::make_shared<BoolLiteralExpression>(token);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
 }
@@ -764,7 +763,7 @@ YaraExpressionBuilder id(const std::string& id)
 {
 	auto ts = std::make_shared<TokenStream>();
 	const std::shared_ptr<Symbol>& symbol = std::make_shared<ValueSymbol>(id, Expression::Type::Object);
-	TokenIt token = ts->emplace_back(ID, std::move(symbol), id);
+	TokenIt token = ts->emplace_back(TokenType::ID, std::move(symbol));
 	auto expression = std::make_shared<IdExpression>(token);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression));
 }
@@ -780,13 +779,13 @@ YaraExpressionBuilder id(const std::string& id)
 YaraExpressionBuilder paren(const YaraExpressionBuilder& other, bool linebreak)
 {
 	auto ts = std::make_shared<TokenStream>();
-	auto lb = ts->emplace_back(LP, "(");
+	auto lb = ts->emplace_back(TokenType::LP, "(");
 	if (linebreak)
-		ts->emplace_back(NEW_LINE, "\n");
-	ts->move_append(other.getTokenStream());
+		ts->emplace_back(TokenType::NEW_LINE, "\n");
+	ts->moveAppend(other.getTokenStream());
 	if (linebreak)
-		ts->emplace_back(NEW_LINE, "\n");
-	auto rb = ts->emplace_back(RP, ")");
+		ts->emplace_back(TokenType::NEW_LINE, "\n");
+	auto rb = ts->emplace_back(TokenType::RP, ")");
 	auto expression = std::make_shared<ParenthesesExpression>(lb, other.get(), rb, linebreak);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), other.getType());
 }
@@ -801,7 +800,7 @@ YaraExpressionBuilder paren(const YaraExpressionBuilder& other, bool linebreak)
 YaraExpressionBuilder stringRef(const std::string& id)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(STRING_ID, id);
+	TokenIt token = ts->emplace_back(TokenType::STRING_ID, id);
 	if (endsWith(id, '*'))
 		return YaraExpressionBuilder(std::move(ts), std::make_shared<StringWildcardExpression>(token));
 	else
@@ -824,7 +823,7 @@ YaraExpressionBuilder matchCount(const std::string& id)
 	countId[0] = '#';
 
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(STRING_COUNT, std::move(countId));
+	TokenIt token = ts->emplace_back(TokenType::STRING_COUNT, std::move(countId));
 	auto expression = std::make_shared<StringCountExpression>(token);
 
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Int);
@@ -846,7 +845,7 @@ YaraExpressionBuilder matchLength(const std::string& id)
 	lengthId[0] = '!';
 
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(STRING_LENGTH, std::move(lengthId));
+	TokenIt token = ts->emplace_back(TokenType::STRING_LENGTH, std::move(lengthId));
 	auto expression = std::make_shared<StringLengthExpression>(token);
 
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Int);
@@ -868,7 +867,7 @@ YaraExpressionBuilder matchOffset(const std::string& id)
 	offsetId[0] = '@';
 
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(STRING_OFFSET, std::move(offsetId));
+	TokenIt token = ts->emplace_back(TokenType::STRING_OFFSET, std::move(offsetId));
 	auto expression = std::make_shared<StringOffsetExpression>(token);
 
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Int);
@@ -891,11 +890,11 @@ YaraExpressionBuilder matchLength(const std::string& id, const YaraExpressionBui
 	lengthId[0] = '!';
 
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(STRING_LENGTH, std::move(lengthId));
+	TokenIt token = ts->emplace_back(TokenType::STRING_LENGTH, std::move(lengthId));
 	auto other_expression = other.get();
-	ts->emplace_back(LSQB, "[");
-	ts->move_append(other_expression->getTokenStream());
-	ts->emplace_back(RSQB, "]");
+	ts->emplace_back(TokenType::LSQB, "[");
+	ts->moveAppend(other_expression->getTokenStream());
+	ts->emplace_back(TokenType::RSQB, "]");
 	auto expression = std::make_shared<StringLengthExpression>(token, std::move(other_expression));
 
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Int);
@@ -918,11 +917,11 @@ YaraExpressionBuilder matchOffset(const std::string& id, const YaraExpressionBui
 	offsetId[0] = '@';
 
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(STRING_OFFSET, std::move(offsetId));
+	TokenIt token = ts->emplace_back(TokenType::STRING_OFFSET, std::move(offsetId));
 	auto other_expression = other.get();
-	ts->emplace_back(LSQB, "[");
-	ts->move_append(other_expression->getTokenStream());
-	ts->emplace_back(RSQB, "]");
+	ts->emplace_back(TokenType::LSQB, "[");
+	ts->moveAppend(other_expression->getTokenStream());
+	ts->emplace_back(TokenType::RSQB, "]");
 	auto expression = std::make_shared<StringOffsetExpression>(token, std::move(other_expression));
 
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Int);
@@ -939,11 +938,11 @@ YaraExpressionBuilder matchOffset(const std::string& id, const YaraExpressionBui
 YaraExpressionBuilder matchAt(const std::string& id, const YaraExpressionBuilder& other)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt id_token = ts->emplace_back(STRING_ID, id);
-	TokenIt at_symbol = ts->emplace_back(OP_AT, "at");
+	TokenIt id_token = ts->emplace_back(TokenType::STRING_ID, id);
+	TokenIt at_symbol = ts->emplace_back(TokenType::OP_AT, "at");
 
 	auto other_expression = other.get();
-	ts->move_append(other_expression->getTokenStream());
+	ts->moveAppend(other_expression->getTokenStream());
 
 	auto expression = std::make_shared<StringAtExpression>(id_token, at_symbol, std::move(other_expression));
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
@@ -960,11 +959,11 @@ YaraExpressionBuilder matchAt(const std::string& id, const YaraExpressionBuilder
 YaraExpressionBuilder matchInRange(const std::string& id, const YaraExpressionBuilder& other)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt id_token = ts->emplace_back(STRING_ID, id);
-	TokenIt in_symbol = ts->emplace_back(OP_IN, "in");
+	TokenIt id_token = ts->emplace_back(TokenType::STRING_ID, id);
+	TokenIt in_symbol = ts->emplace_back(TokenType::OP_IN, "in");
 
 	auto other_expression = other.get();
-	ts->move_append(other_expression->getTokenStream());
+	ts->moveAppend(other_expression->getTokenStream());
 	auto expression = std::make_shared<StringInRangeExpression>(id_token, in_symbol, std::move(other_expression));
 
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
@@ -983,15 +982,15 @@ YaraExpressionBuilder matchInRange(const std::string& id, const YaraExpressionBu
 YaraExpressionBuilder forLoop(const YaraExpressionBuilder& forExpr, const std::string& id, const YaraExpressionBuilder& set, const YaraExpressionBuilder& expr)
 {
 	auto ts = std::make_shared<TokenStream>();
-	auto forToken = ts->emplace_back(FOR, "for");
-	ts->move_append(forExpr.getTokenStream());
-	auto idToken = ts->emplace_back(ID, id);
-	auto inToken = ts->emplace_back(OP_IN, "in");
-	ts->move_append(set.getTokenStream());
-	ts->emplace_back(COLON, ":");
-	auto lb = ts->emplace_back(LP_WITH_SPACE_AFTER, "(");
-	ts->move_append(expr.getTokenStream());
-	auto rb = ts->emplace_back(RP_WITH_SPACE_BEFORE, ")");
+	auto forToken = ts->emplace_back(TokenType::FOR, "for");
+	ts->moveAppend(forExpr.getTokenStream());
+	auto idToken = ts->emplace_back(TokenType::ID, id);
+	auto inToken = ts->emplace_back(TokenType::OP_IN, "in");
+	ts->moveAppend(set.getTokenStream());
+	ts->emplace_back(TokenType::COLON, ":");
+	auto lb = ts->emplace_back(TokenType::LP_WITH_SPACE_AFTER, "(");
+	ts->moveAppend(expr.getTokenStream());
+	auto rb = ts->emplace_back(TokenType::RP_WITH_SPACE_BEFORE, ")");
 
 	auto expression = std::make_shared<ForIntExpression>(forToken, forExpr.get(), idToken, inToken, set.get(), lb, expr.get(), rb);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
@@ -1009,14 +1008,14 @@ YaraExpressionBuilder forLoop(const YaraExpressionBuilder& forExpr, const std::s
 YaraExpressionBuilder forLoop(const YaraExpressionBuilder& forExpr, const YaraExpressionBuilder& set, const YaraExpressionBuilder& expr)
 {
 	auto ts = std::make_shared<TokenStream>();
-	auto forToken = ts->emplace_back(FOR, "for");
-	ts->move_append(forExpr.getTokenStream());
-	auto ofToken = ts->emplace_back(OF, "of");
-	ts->move_append(set.getTokenStream());
-	ts->emplace_back(COLON, ":");
-	auto lb = ts->emplace_back(LP_WITH_SPACE_AFTER, "(");
-	ts->move_append(expr.getTokenStream());
-	auto rb = ts->emplace_back(RP_WITH_SPACE_BEFORE, ")");
+	auto forToken = ts->emplace_back(TokenType::FOR, "for");
+	ts->moveAppend(forExpr.getTokenStream());
+	auto ofToken = ts->emplace_back(TokenType::OF, "of");
+	ts->moveAppend(set.getTokenStream());
+	ts->emplace_back(TokenType::COLON, ":");
+	auto lb = ts->emplace_back(TokenType::LP_WITH_SPACE_AFTER, "(");
+	ts->moveAppend(expr.getTokenStream());
+	auto rb = ts->emplace_back(TokenType::RP_WITH_SPACE_BEFORE, ")");
 
 	auto expression = std::make_shared<ForStringExpression>(forToken, forExpr.get(), ofToken, set.get(), lb, expr.get(), rb);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
@@ -1033,9 +1032,9 @@ YaraExpressionBuilder forLoop(const YaraExpressionBuilder& forExpr, const YaraEx
 YaraExpressionBuilder of(const YaraExpressionBuilder& ofExpr, const YaraExpressionBuilder& set)
 {
 	auto ts = std::make_shared<TokenStream>();
-	ts->move_append(ofExpr.getTokenStream());
-	auto ofToken = ts->emplace_back(OF, "of");
-	ts->move_append(set.getTokenStream());
+	ts->moveAppend(ofExpr.getTokenStream());
+	auto ofToken = ts->emplace_back(TokenType::OF, "of");
+	ts->moveAppend(set.getTokenStream());
 
 	auto expression = std::make_shared<OfExpression>(ofExpr.get(), ofToken, set.get());
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
@@ -1051,14 +1050,14 @@ YaraExpressionBuilder of(const YaraExpressionBuilder& ofExpr, const YaraExpressi
 YaraExpressionBuilder set(const std::vector<YaraExpressionBuilder>& elements)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt lb = ts->emplace_back(LP, "(");
+	TokenIt lb = ts->emplace_back(TokenType::LP, "(");
 	for (std::size_t i = 0; i < elements.size(); ++i)
 	{
-		ts->move_append(elements[i].getTokenStream());
+		ts->moveAppend(elements[i].getTokenStream());
 		if (i < elements.size() - 1)
-			ts->emplace_back(COMMA, ",");
+			ts->emplace_back(TokenType::COMMA, ",");
 	}
-	TokenIt rb = ts->emplace_back(RP, ")");
+	TokenIt rb = ts->emplace_back(TokenType::RP, ")");
 
 	std::vector<Expression::Ptr> elementsExprs;
 	std::for_each(elements.begin(), elements.end(), [&elementsExprs](const YaraExpressionBuilder& expr) { elementsExprs.push_back(expr.get()); });
@@ -1084,11 +1083,11 @@ YaraExpressionBuilder conjunction(const YaraExpressionBuilder& lhs, const YaraEx
 		error_handle(ArgType::Right, "and", "bool", lhs.get());
 
 	auto ts = std::make_shared<TokenStream>();
-	ts->move_append(lhs.getTokenStream());
-	TokenIt andToken = ts->emplace_back(AND, "and");
+	ts->moveAppend(lhs.getTokenStream());
+	TokenIt andToken = ts->emplace_back(TokenType::AND, "and");
 	if (linebreak)
-		ts->emplace_back(NEW_LINE, "\n");
-	ts->move_append(rhs.getTokenStream());
+		ts->emplace_back(TokenType::NEW_LINE, "\n");
+	ts->moveAppend(rhs.getTokenStream());
 
 	auto expression = std::make_shared<AndExpression>(lhs.get(), andToken, rhs.get(), linebreak);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
@@ -1111,11 +1110,11 @@ YaraExpressionBuilder disjunction(const YaraExpressionBuilder& lhs, const YaraEx
 		error_handle(ArgType::Right, "or", "bool", rhs.get());
 
 	auto ts = std::make_shared<TokenStream>();
-	ts->move_append(lhs.getTokenStream());
-	TokenIt orToken = ts->emplace_back(OR, "or");
+	ts->moveAppend(lhs.getTokenStream());
+	TokenIt orToken = ts->emplace_back(TokenType::OR, "or");
 	if (linebreak)
-		ts->emplace_back(NEW_LINE, "\n");
-	ts->move_append(rhs.getTokenStream());
+		ts->emplace_back(TokenType::NEW_LINE, "\n");
+	ts->moveAppend(rhs.getTokenStream());
 
 	auto expression = std::make_shared<OrExpression>(lhs.get(), orToken, rhs.get(), linebreak);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression), Expression::Type::Bool);
@@ -1206,11 +1205,11 @@ YaraExpressionBuilder disjunction(const std::vector<std::pair<YaraExpressionBuil
 YaraExpressionBuilder range(const YaraExpressionBuilder& low, const YaraExpressionBuilder& high)
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt lb = ts->emplace_back(LP, "(");
-	ts->move_append(low.getTokenStream());
-	TokenIt opToken = ts->emplace_back(DOUBLE_DOT, "..");
-	ts->move_append(high.getTokenStream());
-	TokenIt rb = ts->emplace_back(RP, ")");
+	TokenIt lb = ts->emplace_back(TokenType::LP, "(");
+	ts->moveAppend(low.getTokenStream());
+	TokenIt opToken = ts->emplace_back(TokenType::DOUBLE_DOT, "..");
+	ts->moveAppend(high.getTokenStream());
+	TokenIt rb = ts->emplace_back(TokenType::RP, ")");
 
 	auto expression = std::make_shared<RangeExpression>(lb, low.get(), opToken, high.get(), rb);
 	return YaraExpressionBuilder(std::move(ts), std::move(expression));
@@ -1224,7 +1223,7 @@ YaraExpressionBuilder range(const YaraExpressionBuilder& low, const YaraExpressi
 YaraExpressionBuilder filesize()
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(FILESIZE, "filesize");
+	TokenIt token = ts->emplace_back(TokenType::FILESIZE, "filesize");
 	return YaraExpressionBuilder(std::move(ts), std::make_shared<FilesizeExpression>(token), Expression::Type::Int);
 }
 
@@ -1236,7 +1235,7 @@ YaraExpressionBuilder filesize()
 YaraExpressionBuilder entrypoint()
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(ENTRYPOINT, "entrypoint");
+	TokenIt token = ts->emplace_back(TokenType::ENTRYPOINT, "entrypoint");
 	return YaraExpressionBuilder(std::move(ts), std::make_shared<EntrypointExpression>(token), Expression::Type::Int);
 }
 
@@ -1248,7 +1247,7 @@ YaraExpressionBuilder entrypoint()
 YaraExpressionBuilder all()
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(ALL, "all");
+	TokenIt token = ts->emplace_back(TokenType::ALL, "all");
 	return YaraExpressionBuilder(std::move(ts), std::make_shared<AllExpression>(token));
 }
 
@@ -1260,7 +1259,7 @@ YaraExpressionBuilder all()
 YaraExpressionBuilder any()
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(ANY, "any");
+	TokenIt token = ts->emplace_back(TokenType::ANY, "any");
 	return YaraExpressionBuilder(std::move(ts), std::make_shared<AnyExpression>(token));
 }
 
@@ -1272,7 +1271,7 @@ YaraExpressionBuilder any()
 YaraExpressionBuilder them()
 {
 	auto ts = std::make_shared<TokenStream>();
-	TokenIt token = ts->emplace_back(THEM, "them");
+	TokenIt token = ts->emplace_back(TokenType::THEM, "them");
 	return YaraExpressionBuilder(std::move(ts), std::make_shared<ThemExpression>(token));
 }
 
