@@ -14,6 +14,9 @@ using namespace yaramod;
 
 void addVisitorClasses(py::module& module)
 {
+	py::class_<TokenStreamContext>(module, "TokenStreamContext")
+		.def(py::init<Expression*>());
+
 	py::class_<Visitor, PyVisitor>(module, "Visitor")
 		.def(py::init<>())
 		.def("visit_StringExpression", py::overload_cast<StringExpression*>(&Visitor::visit))
@@ -126,6 +129,7 @@ void addVisitorClasses(py::module& module)
 	py::class_<ModifyingVisitor, PyModifyingVisitor, Visitor>(module, "ModifyingVisitor")
 		.def(py::init<>())
 		.def("modify", &ModifyingVisitor::modify, py::arg("expr"), py::arg("when_deleted") = static_cast<Expression*>(nullptr))
+		.def("cleanup_tokenstreams", &ModifyingVisitor::cleanUpTokenStreams, py::arg("context"), py::arg("new_expression"))
 		.def("visit_StringExpression", py::overload_cast<StringExpression*>(&ModifyingVisitor::visit))
 		.def("visit_StringWildcardExpression", py::overload_cast<StringWildcardExpression*>(&ModifyingVisitor::visit))
 		.def("visit_StringAtExpression", py::overload_cast<StringAtExpression*>(&ModifyingVisitor::visit))
@@ -177,43 +181,43 @@ void addVisitorClasses(py::module& module)
 		.def("visit_ParenthesesExpression", py::overload_cast<ParenthesesExpression*>(&ModifyingVisitor::visit))
 		.def("visit_IntFunctionExpression", py::overload_cast<IntFunctionExpression*>(&ModifyingVisitor::visit))
 		.def("visit_RegexpExpression", py::overload_cast<RegexpExpression*>(&ModifyingVisitor::visit))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(StringAtExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(StringInRangeExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(StringOffsetExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(StringLengthExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(NotExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(UnaryMinusExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(BitwiseNotExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(AndExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(OrExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(LtExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(GtExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(LeExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(GeExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(EqExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(NeqExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(ContainsExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(MatchesExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(PlusExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(MinusExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(MultiplyExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(DivideExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(ModuloExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(BitwiseXorExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(BitwiseAndExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(BitwiseOrExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(ShiftLeftExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(ShiftRightExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(ForIntExpression*, const VisitResult&, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(ForStringExpression*, const VisitResult&, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(OfExpression*, const VisitResult&, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(SetExpression*, const std::vector<VisitResult>&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(RangeExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(StructAccessExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(ArrayAccessExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(FunctionCallExpression*, const VisitResult&, const std::vector<VisitResult>&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(ParenthesesExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
-		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(IntFunctionExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler));
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, StringAtExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, StringInRangeExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, StringOffsetExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, StringLengthExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, NotExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, UnaryMinusExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, BitwiseNotExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, AndExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, OrExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, LtExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, GtExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, LeExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, GeExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, EqExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, NeqExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, ContainsExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, MatchesExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, PlusExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, MinusExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, MultiplyExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, DivideExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, ModuloExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, BitwiseXorExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, BitwiseAndExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, BitwiseOrExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, ShiftLeftExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, ShiftRightExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, ForIntExpression*, const VisitResult&, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, ForStringExpression*, const VisitResult&, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, OfExpression*, const VisitResult&, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, SetExpression*, const std::vector<VisitResult>&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, RangeExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, StructAccessExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, ArrayAccessExpression*, const VisitResult&, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, FunctionCallExpression*, const VisitResult&, const std::vector<VisitResult>&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, ParenthesesExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler))
+		.def("default_handler", static_cast<VisitResult(ModifyingVisitor::*)(const TokenStreamContext&, IntFunctionExpression*, const VisitResult&)>(&ModifyingVisitor::defaultHandler));
 }
 
 void addRegexpVisitorClasses(py::module& module)
