@@ -15,6 +15,7 @@
 #include <yaramod/builder/yara_hex_string_builder.h>
 #include <yaramod/builder/yara_rule_builder.h>
 #include <yaramod/types/plain_string.h>
+#include <yaramod/types/token_type.h>
 #include <yaramod/yaramod.h>
 
 #include "yaramod_python.h"
@@ -108,6 +109,135 @@ void addEnums(py::module& module)
 
 	py::enum_<VisitAction>(module, "VisitAction")
 		.value("Delete", VisitAction::Delete);
+
+	py::enum_<yaramod::TokenType>(module, "TokenType")
+		.value("RuleName", TokenType::RULE_NAME)
+		.value("Tag", TokenType::TAG)
+		.value("HexAlt", TokenType::HEX_ALT)
+		.value("HexNibble", TokenType::HEX_NIBBLE)
+		.value("HexWildcard", TokenType::HEX_WILDCARD)
+		.value("HexWildcardLow", TokenType::HEX_WILDCARD_LOW)
+		.value("HexWildcardHigh", TokenType::HEX_WILDCARD_HIGH)
+		.value("HexJumpLeftBracket", TokenType::HEX_JUMP_LEFT_BRACKET)
+		.value("HexJumpRightBracket", TokenType::HEX_JUMP_RIGHT_BRACKET)
+		.value("HexAltLeftBracket", TokenType::HEX_ALT_LEFT_BRACKET)
+		.value("HexAltRightBracket", TokenType::HEX_ALT_RIGHT_BRACKET)
+		.value("HexJumpFixed", TokenType::HEX_JUMP_FIXED)
+		.value("HexStartBracket", TokenType::HEX_START_BRACKET)
+		.value("HexEndBracket", TokenType::HEX_END_BRACKET)
+		.value("NewLine", TokenType::NEW_LINE)
+		.value("Meta", TokenType::META)
+		.value("Lquote", TokenType::LQUOTE)
+		.value("Rquote", TokenType::RQUOTE)
+		.value("RuleEnd", TokenType::RULE_END)
+		.value("RuleBegin", TokenType::RULE_BEGIN)
+		.value("Range", TokenType::RANGE)
+		.value("Dot", TokenType::DOT)
+		.value("DoubleDot", TokenType::DOUBLE_DOT)
+		.value("Lt", TokenType::LT)
+		.value("Gt", TokenType::GT)
+		.value("Le", TokenType::LE)
+		.value("Ge", TokenType::GE)
+		.value("Eq", TokenType::EQ)
+		.value("Neq", TokenType::NEQ)
+		.value("ShiftLeft", TokenType::SHIFT_LEFT)
+		.value("ShiftRight", TokenType::SHIFT_RIGHT)
+		.value("Minus", TokenType::MINUS)
+		.value("Plus", TokenType::PLUS)
+		.value("Multiply", TokenType::MULTIPLY)
+		.value("Divide", TokenType::DIVIDE)
+		.value("Modulo", TokenType::MODULO)
+		.value("BitwiseXor", TokenType::BITWISE_XOR)
+		.value("BitwiseAnd", TokenType::BITWISE_AND)
+		.value("BitwiseOr", TokenType::BITWISE_OR)
+		.value("BitwiseNot", TokenType::BITWISE_NOT)
+		.value("Lp", TokenType::LP)
+		.value("Rp", TokenType::RP)
+		.value("Lcb", TokenType::LCB)
+		.value("Rcb", TokenType::RCB)
+		.value("Assign", TokenType::ASSIGN)
+		.value("Colon", TokenType::COLON)
+		.value("ColonBeforeNewline", TokenType::COLON_BEFORE_NEWLINE)
+		.value("Comma", TokenType::COMMA)
+		.value("Private", TokenType::PRIVATE)
+		.value("Global", TokenType::GLOBAL)
+		.value("None", TokenType::NONE)
+		.value("Rule", TokenType::RULE)
+		.value("Strings", TokenType::STRINGS)
+		.value("Condition", TokenType::CONDITION)
+		.value("Ascii", TokenType::ASCII)
+		.value("Nocase", TokenType::NOCASE)
+		.value("Wide", TokenType::WIDE)
+		.value("Fullword", TokenType::FULLWORD)
+		.value("PrivateStringModifier", TokenType::PRIVATE_STRING_MODIFIER)
+		.value("Xor", TokenType::XOR)
+		.value("ImportModule", TokenType::IMPORT_MODULE)
+		.value("ImportKeyword", TokenType::IMPORT_KEYWORD)
+		.value("Not", TokenType::NOT)
+		.value("And", TokenType::AND)
+		.value("Or", TokenType::OR)
+		.value("All", TokenType::ALL)
+		.value("Any", TokenType::ANY)
+		.value("Of", TokenType::OF)
+		.value("Them", TokenType::THEM)
+		.value("For", TokenType::FOR)
+		.value("Entrypoint", TokenType::ENTRYPOINT)
+		.value("OpAt", TokenType::OP_AT)
+		.value("OpIn", TokenType::OP_IN)
+		.value("Filesize", TokenType::FILESIZE)
+		.value("Contains", TokenType::CONTAINS)
+		.value("Matches", TokenType::MATCHES)
+		.value("Slash", TokenType::SLASH)
+		.value("StringLiteral", TokenType::STRING_LITERAL)
+		.value("Integer", TokenType::INTEGER)
+		.value("Double", TokenType::DOUBLE)
+		.value("StringId", TokenType::STRING_ID)
+		.value("StringIdAfterNewline", TokenType::STRING_ID_AFTER_NEWLINE)
+		.value("StringIdWildcard", TokenType::STRING_ID_WILDCARD)
+		.value("StringLength", TokenType::STRING_LENGTH)
+		.value("StringOffset", TokenType::STRING_OFFSET)
+		.value("StringCount", TokenType::STRING_COUNT)
+		.value("Id", TokenType::ID)
+		.value("IntegerFunction", TokenType::INTEGER_FUNCTION)
+		.value("Lsqb", TokenType::LSQB)
+		.value("Rsqb", TokenType::RSQB)
+		.value("Dash", TokenType::DASH)
+		.value("RegexpOr", TokenType::REGEXP_OR)
+		.value("RegexpIter", TokenType::REGEXP_ITER)
+		.value("RegexpPiter", TokenType::REGEXP_PITER)
+		.value("RegexpOptional", TokenType::REGEXP_OPTIONAL)
+		.value("RegexpStartSlash", TokenType::REGEXP_START_SLASH)
+		.value("RegexpEndSlash", TokenType::REGEXP_END_SLASH)
+		.value("RegexpChar", TokenType::REGEXP_CHAR)
+		.value("RegexpRange", TokenType::REGEXP_RANGE)
+		.value("RegexpText", TokenType::REGEXP_TEXT)
+		.value("RegexpClassNegative", TokenType::REGEXP_CLASS_NEGATIVE)
+		.value("RegexpModifiers", TokenType::REGEXP_MODIFIERS)
+		.value("RegexpGreedy", TokenType::REGEXP_GREEDY)
+		.value("UnaryMinus", TokenType::UNARY_MINUS)
+		.value("MetaKey", TokenType::META_KEY)
+		.value("MetaValue", TokenType::META_VALUE)
+		.value("StringKey", TokenType::STRING_KEY)
+		.value("ValueSymbol", TokenType::VALUE_SYMBOL)
+		.value("FunctionSymbol", TokenType::FUNCTION_SYMBOL)
+		.value("ArraySymbol", TokenType::ARRAY_SYMBOL)
+		.value("DictionarySymbol", TokenType::DICTIONARY_SYMBOL)
+		.value("StructureSymbol", TokenType::STRUCTURE_SYMBOL)
+		.value("LpEnumeration", TokenType::LP_ENUMERATION)
+		.value("RpEnumeration", TokenType::RP_ENUMERATION)
+		.value("LpWithSpaceAfter", TokenType::LP_WITH_SPACE_AFTER)
+		.value("RpWithSpaceBefore", TokenType::RP_WITH_SPACE_BEFORE)
+		.value("LpWithSpaces", TokenType::LP_WITH_SPACES)
+		.value("RpWithSpaces", TokenType::RP_WITH_SPACES)
+		.value("BoolTrue", TokenType::BOOL_TRUE)
+		.value("BoolFalse", TokenType::BOOL_FALSE)
+		.value("OnelineComment", TokenType::ONELINE_COMMENT)
+		.value("Comment", TokenType::COMMENT)
+		.value("IncludeDirective", TokenType::INCLUDE_DIRECTIVE)
+		.value("IncludePath", TokenType::INCLUDE_PATH)
+		.value("FunctionCallLp", TokenType::FUNCTION_CALL_LP)
+		.value("FunctionCallRp", TokenType::FUNCTION_CALL_RP)
+		.value("Invalid", TokenType::INVALID);
 }
 
 void addBasicClasses(py::module& module)
@@ -117,8 +247,11 @@ void addBasicClasses(py::module& module)
 		.def_property_readonly("rules", &YaraFile::getRules)
 		.def_property_readonly("imports", &YaraFile::getImports)
 		.def_property_readonly("text_formatted", [](const YaraFile& self) { return self.getTextFormatted(); })
+		.def_property_readonly("tokenstream", [](const YaraFile& self) { return self.getTokenStream();} )
 		.def("find_symbol", &YaraFile::findSymbol)
-		.def("add_rule", py::overload_cast<const std::shared_ptr<Rule>&>(&YaraFile::addRule))
+		.def("add_rule", [](YaraFile& self, const std::shared_ptr<Rule>& rule) {
+				self.addRule(rule, true);
+			})
 		.def("insert_rule", py::overload_cast<std::size_t, const std::shared_ptr<Rule>&>(&YaraFile::insertRule))
 		.def("remove_rules", [](YaraFile& self, const std::function<bool(const std::shared_ptr<Rule>&)>& pred) {
 				self.removeRules(pred);
@@ -127,29 +260,43 @@ void addBasicClasses(py::module& module)
 				self.removeImports(pred);
 			});
 
+	py::class_<Location>(module, "Location")
+		.def_property_readonly("file_path", &Location::getFilePath)
+		// `line_number` property is deprecated, preferred way is to use `begin.line`
+		.def_property_readonly("line_number", [](const Location& self) { return self.begin().getLine(); })
+		.def_property_readonly("begin", &Location::begin)
+		.def_property_readonly("end", &Location::end)
+		.def_property_readonly("text", &Location::getText);
+
+	py::class_<Location::Position>(module, "Position")
+		.def_property_readonly("line", &Location::Position::getLine)
+		.def_property_readonly("column", &Location::Position::getColumn);
+
 	py::class_<Rule, std::shared_ptr<Rule>>(module, "Rule")
 		.def_property_readonly("text", &Rule::getText)
 		.def_property("name", &Rule::getName, &Rule::setName)
 		.def_property("metas", py::overload_cast<>(&Rule::getMetas), &Rule::setMetas, py::return_value_policy::reference)
-		.def_property_readonly("strings", &Rule::getStrings, py::return_value_policy::reference)
 		.def_property("tags", &Rule::getTags, &Rule::setTags)
-		.def_property_readonly("modifier", &Rule::getModifier)
+		.def_property("modifier", &Rule::getModifier, &Rule::setModifier)
+		.def_property_readonly("strings", &Rule::getStrings, py::return_value_policy::reference)
 		.def_property_readonly("is_private", &Rule::isPrivate)
 		.def_property_readonly("is_global", &Rule::isGlobal)
 		.def_property_readonly("location", &Rule::getLocation)
 		.def_property_readonly("symbol", &Rule::getSymbol)
+		.def_property_readonly("token_first", [](Rule& self) {
+				return *self.getFirstTokenIt();
+			})
+		.def_property_readonly("token_last", [](Rule& self) {
+				return *self.getLastTokenIt();
+			})
 		.def_property("condition", &Rule::getCondition, &Rule::setCondition)
 		.def("add_meta", &Rule::addMeta)
 		.def("remove_metas", &Rule::removeMetas)
 		.def("remove_string", &Rule::removeString)
-		.def("get_meta_with_name", &Rule::getMetaWithName, py::return_value_policy::reference)
+		.def("get_meta_with_name", py::overload_cast<const std::string&>(&Rule::getMetaWithName), py::return_value_policy::reference)
 		.def("add_tag", &Rule::addTag)
 		.def("remove_tags", py::overload_cast<const std::string&>(&Rule::removeTags));
 
-	py::class_<Rule::Location>(module, "RuleLocation")
-		.def_readonly("file_path", &Rule::Location::filePath)
-		.def_readonly("line_number", &Rule::Location::lineNumber);
-
 	py::class_<Meta>(module, "Meta")
 		.def_property("key", &Meta::getKey, &Meta::setKey)
 		.def_property("value", &Meta::getValue, &Meta::setValue);
@@ -163,7 +310,7 @@ void addBasicClasses(py::module& module)
 		.def(py::init<std::uint64_t, const std::string&>())
 		.def(py::init<double>())
 		.def(py::init<double, const std::string&>())
-		.def(py::init<const std::shared_ptr<Symbol>&, const std::string&>())
+		.def(py::init<const std::shared_ptr<Symbol>&>())
 		.def_property_readonly("text", [](Literal& self) { return self.getText(); })
 		.def_property_readonly("pure_text", &Literal::getPureText)
 		.def_property_readonly("is_string", &Literal::isString)
@@ -197,7 +344,16 @@ void addBasicClasses(py::module& module)
 		.def_property_readonly("is_nocase", &String::isNocase)
 		.def_property_readonly("is_private", &String::isPrivate)
 		.def_property_readonly("is_xor", &String::isXor)
-		.def_property_readonly("modifiers_text", &String::getModifiersText);
+		.def_property_readonly("location", &String::getLocation)
+		.def_property_readonly("modifiers_text", &String::getModifiersText)
+		.def_property_readonly("token_id", &String::getIdentifierToken)
+		.def_property_readonly("token_assign", &String::getAssignToken)
+		.def_property_readonly("token_first", [](String& self) {
+				return *self.getFirstTokenIt();
+			})
+		.def_property_readonly("token_last", [](String& self) {
+				return *self.getLastTokenIt();
+			});
 
 	py::class_<PlainString, String, std::shared_ptr<PlainString>>(module, "PlainString");
 	py::class_<HexString, String, std::shared_ptr<HexString>>(module, "HexString");
@@ -292,16 +448,50 @@ void addBasicClasses(py::module& module)
 			});
 }
 
+void addTokenStreamClass(py::module& module)
+{
+	py::class_<Token>(module, "Token")
+		.def(py::init<yaramod::TokenType, const Literal&>())
+		.def_property_readonly("text", [](Token& self) { return self.getText(); })
+		.def_property_readonly("pure_text", &Token::getPureText)
+		.def_property_readonly("is_string", &Token::isString)
+		.def_property_readonly("is_bool", &Token::isBool)
+		.def_property_readonly("is_int", &Token::isInt)
+		.def_property_readonly("is_float", &Token::isFloat)
+		.def_property_readonly("is_symbol", &Token::isSymbol)
+		.def_property_readonly("type", &Token::getType)
+		.def_property_readonly("literal", &Token::getLiteral)
+		.def_property_readonly("location", &Token::getLocation)
+		.def_property_readonly("string", &Token::getString)
+		.def_property_readonly("bool", &Token::getBool)
+		.def_property_readonly("int", &Token::getInt)
+		.def_property_readonly("uint", &Token::getUInt)
+		.def_property_readonly("float", &Token::getFloat)
+		.def_property_readonly("symbol", &Token::getSymbol)
+		.def_property_readonly("literal_reference", &Token::getLiteralReference);
+
+	py::class_<TokenStream, std::shared_ptr<TokenStream>>(module, "TokenStream")
+		.def(py::init<>())
+		.def_property_readonly("empty", &TokenStream::empty)
+		.def_property_readonly("size", &TokenStream::size)
+		.def_property_readonly("front", &TokenStream::front)
+		.def_property_readonly("back", &TokenStream::back)
+		.def_property_readonly("tokens", &TokenStream::getTokens)
+		.def_property_readonly("tokens_as_text", &TokenStream::getTokensAsText);
+}
+
 void addExpressionClasses(py::module& module)
 {
 	py::class_<Expression, std::shared_ptr<Expression>>(module, "Expression")
 		.def("accept", &Expression::accept)
 		.def("get_text", &Expression::getText, py::arg("indent") = std::string{})
+		.def("exchange_tokens", py::overload_cast<Expression*>(&Expression::exchangeTokens))
 		.def_property_readonly("text",
 				// getText() has default parameter and Python can't deal with it
 				[](const Expression* self) {
 					return self->getText();
-				});
+				})
+		.def_property_readonly("tokenstream", [](const Expression& self) { return self.getTokenStream();} );
 
 	exprClass<StringExpression>(module, "StringExpression")
 		.def_property("id",
@@ -390,7 +580,10 @@ void addExpressionClasses(py::module& module)
 		.def_property("body",
 				&ForExpression::getBody,
 				py::overload_cast<const Expression::Ptr&>(&ForExpression::setBody));
-	exprClass<ForIntExpression, ForExpression>(module, "ForIntExpression");
+	exprClass<ForIntExpression, ForExpression>(module, "ForIntExpression")
+		.def_property("id",
+				&ForIntExpression::getId,
+				py::overload_cast<const std::string&>(&ForIntExpression::setId));
 	exprClass<ForStringExpression, ForExpression>(module, "ForStringExpression");
 	exprClass<OfExpression, ForExpression>(module, "OfExpression");
 
@@ -544,6 +737,7 @@ void addBuilderClasses(py::module& module)
 			})
 		.def("__getitem__", &YaraExpressionBuilder::operator[])
 		.def("access", &YaraExpressionBuilder::access)
+		.def("comment", &YaraExpressionBuilder::comment, py::arg("message"), py::arg("multiline") = false, py::arg("indent") = "")
 		.def("contains", &YaraExpressionBuilder::contains)
 		.def("matches", &YaraExpressionBuilder::matches)
 		.def("read_int8", &YaraExpressionBuilder::readInt8)
@@ -638,7 +832,8 @@ void addMainClass(py::module& module)
 		.def("parse_string", [](Yaramod& self, const std::string& str, ParserMode parserMode) {
 				std::istringstream stream(str);
 				return self.parseStream(stream, parserMode);
-			}, py::arg("str"), py::arg("parser_mode") = ParserMode::Regular);
+			}, py::arg("str"), py::arg("parser_mode") = ParserMode::Regular)
+		.def_property_readonly("yara_file", &Yaramod::getParsedFile);
 }
 
 PYBIND11_MODULE(yaramod, module)
@@ -660,6 +855,7 @@ PYBIND11_MODULE(yaramod, module)
 	addVersionVariables(module);
 	addEnums(module);
 	addBasicClasses(module);
+	addTokenStreamClass(module);
 	addExpressionClasses(module);
 	addMainClass(module);
 	addVisitorClasses(module);
