@@ -210,19 +210,23 @@ bool YaraFile::addImport(TokenIt import, ModulesPool& modules)
  * Adds the rule to the YARA file.
  *
  * @param rule Rule to add.
+ * @param extractTokens set if the tokens should be extracted into the _tokenStream.
  */
-void YaraFile::addRule(Rule&& rule)
+void YaraFile::addRule(Rule&& rule, bool extractTokens)
 {
-	addRule(std::make_unique<Rule>(std::move(rule)));
+	addRule(std::make_unique<Rule>(std::move(rule)), extractTokens);
 }
 
 /**
  * Adds the rule to the YARA file.
  *
  * @param rule Rule to add.
+ * @param extractTokens set if the tokens should be extracted into the _tokenStream.
  */
-void YaraFile::addRule(std::unique_ptr<Rule>&& rule)
+void YaraFile::addRule(std::unique_ptr<Rule>&& rule, bool extractTokens)
 {
+	if (extractTokens && (rule->getTokenStream() != _tokenStream.get()))
+		_tokenStream->moveAppend(rule->getTokenStream());
 	_rules.emplace_back(std::move(rule));
 	_ruleTable.emplace(_rules.back()->getName(), _rules.back().get());
 }
@@ -231,22 +235,26 @@ void YaraFile::addRule(std::unique_ptr<Rule>&& rule)
  * Adds the rule to the YARA file.
  *
  * @param rule Rule to add.
+ * @param extractTokens set if the tokens should be extracted into the _tokenStream.
  */
-void YaraFile::addRule(const std::shared_ptr<Rule>& rule)
+void YaraFile::addRule(const std::shared_ptr<Rule>& rule, bool extractTokens)
 {
+	if (extractTokens && (rule->getTokenStream() != _tokenStream.get()))
+		_tokenStream->moveAppend(rule->getTokenStream());
 	_rules.emplace_back(rule);
-	_ruleTable.emplace(rule->getName(), _rules.back().get());
+	_ruleTable.emplace(_rules.back()->getName(), _rules.back().get());
 }
 
 /**
  * Adds the rules to the YARA file.
  *
  * @param rules Rules to add.
+ * @param extractTokens set if the tokens should be extracted into the _tokenStream.
  */
-void YaraFile::addRules(const std::vector<std::shared_ptr<Rule>>& rules)
+void YaraFile::addRules(const std::vector<std::shared_ptr<Rule>>& rules, bool extractTokens)
 {
 	for (const auto& rule : rules)
-		addRule(rule);
+		addRule(rule, extractTokens);
 }
 
 /**
@@ -277,6 +285,17 @@ bool YaraFile::addImports(const std::vector<TokenIt>& imports, ModulesPool& modu
 void YaraFile::insertRule(std::size_t position, std::unique_ptr<Rule>&& rule)
 {
 	position = std::min(position, _rules.size());
+	TokenIt before;
+	if (position == _rules.size())
+	{
+		before = _tokenStream->end();
+	}
+	else
+	{
+		before = _rules[position]->getFirstTokenIt();
+	}
+	_tokenStream->moveAppend(before, rule->getTokenStream());
+
 	_rules.insert(_rules.begin() + position, std::move(rule));
 	_ruleTable.emplace(_rules[position]->getName(), _rules[position].get());
 }
@@ -290,6 +309,17 @@ void YaraFile::insertRule(std::size_t position, std::unique_ptr<Rule>&& rule)
 void YaraFile::insertRule(std::size_t position, const std::shared_ptr<Rule>& rule)
 {
 	position = std::min(position, _rules.size());
+	TokenIt before;
+	if (position == _rules.size())
+	{
+		before = _tokenStream->end();
+	}
+	else
+	{
+		before = _rules[position]->getFirstTokenIt();
+	}
+	_tokenStream->moveAppend(before, rule->getTokenStream());
+
 	_rules.insert(_rules.begin() + position, rule);
 	_ruleTable.emplace(_rules[position]->getName(), _rules[position].get());
 }
